using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Runtime.InteropServices;
using System.Web.UI;
using System.Web.UI.WebControls.WebParts;
using Microsoft.SharePoint;
using Microsoft.SharePoint.Administration;
using Microsoft.SharePoint.WebPartPages;

namespace RH.SitecollectionList
{
	[Guid("90710b49-d6b2-40c1-838c-0686dd0a0b84")]
	public class RHSitecollectionList : Microsoft.SharePoint.WebPartPages.WebPart
	{
		private readonly Version _Version = new Version(1, 0, 1, 0);
		private string _ErrorMessage = string.Empty;

		[Personalizable(PersonalizationScope.Shared)]
		[WebBrowsable(true)]
		[Category("Properties")]
		[WebDisplayName("all Webapplications?")]
		[WebDescription("Show Sitecollections from all Webapplications?")]
		public bool AllWebapplications { get; set; }

		[Personalizable(PersonalizationScope.Shared)]
		[WebBrowsable(true)]
		[Category("Properties")]
		[WebDisplayName("show subwebs instead of sitecollections")]
		[WebDescription("Show subwebs of the current web, instead of showing all sitecollections from the current webapplication")]
		public bool UseSubwebs { get; set; }

		public Version Version
		{
			get { return _Version; }
		}

		public override ToolPart[] GetToolParts()
		{
			try
			{
				var toolparts = new ToolPart[3];
				var header = new CustomToolPart();
				var wptp = new WebPartToolPart();
				var footer = new CustomPropertyToolPart();
				toolparts[0] = header;
				toolparts[1] = wptp;
				toolparts[2] = footer;

				// This is the custom ToolPart.
				toolparts[0] = new CustomToolPart();
				toolparts[2] = new CustomToolPartFooter();

				return toolparts;
			}
			catch (Exception ex)
			{
				_ErrorMessage += string.Format("Error in ToolPart: {0}", Common.GetErrorMessage(ex));
			}
			return null;
		}

		/// <summary>
		/// Create all your controls here for rendering.
		/// Try to avoid using the RenderWebPart() method.
		/// </summary>
		protected override void CreateChildControls()
		{
			try
			{
				base.CreateChildControls();

				// read subwebs or sitecollections
				List<NavigationObjects> navigationObjects = UseSubwebs ? GetSubWebs() : GetSitecollections();

				// order by title
				navigationObjects.Sort();

				// holds the links
				var links = new Links();
				Controls.Add(links);

				foreach (NavigationObjects objects in navigationObjects)
				{
					links.AddHyperLink(objects.Title, objects.Description, objects.Url, objects.ImageUrl);
				}

			}
			catch (Exception ex)
			{
				HandleException(ex);
			}
		}

		/// <summary>
		/// Clear all child controls and add an error message for display.
		/// </summary>
		/// <param name="ex"></param>
		private void HandleException(Exception ex)
		{
			Controls.Clear();
			Controls.Add(new LiteralControl(ex.Message));
		}

		/// <summary>
		/// Get sites which the current user may access
		/// </summary>
		/// <returns></returns>
		private List<NavigationObjects> GetSitecollections()
		{
			SPUser currentLogin = SPContext.Current.Web.CurrentUser;
			var sitecollections = new List<NavigationObjects>();

			// Connect to the site as systemaccount
			SPSite currentSite = SPContext.Current.Site;
			using (var site = new SPSite(currentSite.ID, currentSite.SystemAccount.UserToken))
			{
				SPWebApplication webApplication = site.WebApplication;
				if (AllWebapplications)
				{
					SPFarm farm = webApplication.Farm;
					var service = farm.Services.GetValue<SPWebService>("");
					foreach (SPWebApplication application in service.WebApplications)
					{
						GetSitecollectionsFromWebapplication(application, currentLogin, sitecollections);
					}
				}
				else
				{
					GetSitecollectionsFromWebapplication(webApplication, currentLogin, sitecollections);
				}
			}
			return sitecollections;
		}

		private void GetSitecollectionsFromWebapplication(SPWebApplication webApplication, SPUser currentLogin, ICollection<NavigationObjects> sitecollections)
		{
			foreach (SPSite subSite in webApplication.Sites)
			{
				bool catchAccessDenied = subSite.CatchAccessDeniedException;
				try
				{
					subSite.CatchAccessDeniedException = false;
					using (var rootWeb = subSite.RootWeb)
					{
						// check if the current user
						try
						{
							if (rootWeb.DoesUserHavePermissions(currentLogin.LoginName, SPBasePermissions.Open))
							{
								var newSite = new NavigationObjects
														{
															Title = rootWeb.Title,
															Description = rootWeb.Description,
															Url = rootWeb.Url,
															ImageUrl = rootWeb.SiteLogoUrl
														};
								sitecollections.Add(newSite);
							}
						}
						catch (Exception)
						{
							// the user does not have access to the site
						}
					}
				}
				catch (Exception ex)
				{
					HandleException(ex);
				}
				finally
				{
					subSite.CatchAccessDeniedException = catchAccessDenied;
					subSite.Dispose();
				}
			}
		}

		private static List<NavigationObjects> GetSubWebs()
		{
			var subWebs = new List<NavigationObjects>();
			foreach (SPWeb subWeb in SPContext.Current.Web.GetSubwebsForCurrentUser())
			{
				var newSite = new NavigationObjects
				{
					Title = subWeb.Title,
					Description = subWeb.Description,
					Url = subWeb.ServerRelativeUrl,
					ImageUrl = subWeb.SiteLogoUrl
				};
				subWebs.Add(newSite);
			}
			return subWebs;
		}
	}
}